package org.skduemu.service;

import java.io.*;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.Marshaller;
import jakarta.xml.bind.Unmarshaller;
import org.skduemu.entity.Sender;
import org.skduemu.entity.SubscribeEvent;
import org.skduemu.repository.EventRepository;
import org.skduemu.xml.entity.*;
import org.skduemu.xml.entity.Error;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import jakarta.annotation.PostConstruct;

@Service
public class DeviceService {

	private Logger log = Logger.getLogger("DeviceService");

	@Value("${deviceIp:192.168.1.28}")
	private String host;

	@Value("${udpIp:10.0.96.255}")
	private String broadcastIp;

	@Value("${devicePort:19001}")
	private int port;

	@Value("${errorPort:19002}")
	private int errorPort;

	@Value("${serviceFilePath:null}")
	private String serviceFilePath;

	@Value("${deviceFilePath:null}")
	private String deviceFilePath;

	@Value("${eventForSub:null}")
	private String eventForSub;

	@Value("${serviceConfigFilePath:null}")
	private String serviceConfigFilePath;

	@Value("${serviceWebFilePath:null}")
	private String serviceWebFilePath;

	@Value("${serviceMapFilePath:null}")
	private String serviceMapFilePath;

	@Value("${eventForCheckState:null}")
	private String eventForCheckState;

	@Value("${notErrorMessagePath:null}")
	private String notErrorMessagePath;

	@Autowired
	private EventRepository eventRepository;

	byte[] buffer = new byte[4096]; // Буфер для получения данных

	private Map<String, Thread> subscriptions = new HashMap<>();

	private Boolean notSolved = true;

	public void createConnection() {

		try (ServerSocket serverSocket = new ServerSocket(port)) {
			System.out.println("Сервер запущен, ожидаем подключения...");

			while (true) {
				Socket socket = serverSocket.accept();
				System.out.println("Клиент подключился: " + socket.getInetAddress());

				// Поток для получения данных
				InputStream input = socket.getInputStream();
				BufferedReader reader = new BufferedReader(new InputStreamReader(input));

				StringBuilder message = new StringBuilder();
				String line = null;
				while ((line = reader.readLine()) != null) {
					if (line.contains("</get>")) {
						message.append(line).append("\n");
						break; // Завершаем чтение
					}
					if (line.contains("</body>")) {
						message.append(line).append("\n");
						break; // Завершаем чтение
					}
					if (line.contains("</event>")) {
						message.append(line).append("\n");
						break; // Завершаем чтение
					}
					if (line.equals("<event urn=\"KSK-MSUEV-04b-003d:2\"/>")) {
						message.append(line).append("\n");
						break; // Завершаем чтение
					}
					if (line.contains("</error>")) {
						message.append(line).append("\n");
						break; // Завершаем чтение
					}
					message.append(line).append("\n");
				}

				log.info("Сообщение от клиента: " + message.toString());


				OutputStream output = socket.getOutputStream();
				PrintWriter writer = new PrintWriter(output, true);

				getResponseType(message.toString(), writer);

			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	// Распределяем unmarshall методы по классам из запроса
	public void getResponseType(String message, PrintWriter writer) {
		if (message.contains("<event urn=")) {
			unmarshallEvent(message, writer);
		} else if (message.contains("<get>")) {
			unmarshallGet(message, writer);
		} else if (message.contains("<body urn=")) {
			unmarshallBody(message);
		} else if (message.contains("<error urn=") & message.contains("<detail>")){
			unmarshallError(message, writer);
		}
	}

	public void unmarshallError(String message, PrintWriter printWriter){
		try{
			Error error = generalUnmarshalling(Error.class, message);
			//тут должны быть обработка кода, пришедшей ошибки, но ее нет :D
			File xmlFile = new File(notErrorMessagePath);
			Error response = generalUnmarshalling(Error.class, xmlFile);
			Marshaller marshaller = generalMarshalling(Error.class);
			StringWriter stringWriter = new StringWriter();
			marshaller.marshal(response, stringWriter);
			printWriter.println(stringWriter);
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}

	public void unmarshallBody(String message) {
		try {
			Body body = generalUnmarshalling(Body.class, message);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Marshaller generalMarshalling(Class object){
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(object);
			Marshaller marshaller = jaxbContext.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			return marshaller;
		}
		catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}

	public <T> T generalUnmarshalling(Class<T> object, String message){
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(object);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			T result = (T) jaxbUnmarshaller.unmarshal(new StringReader(message));
			return result;
		}
		catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}
	public <T> T generalUnmarshalling(Class<T> object, File file){
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(object);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			T result = (T) jaxbUnmarshaller.unmarshal(file);
			return result;
		}
		catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}



	public void unmarshallEvent(String message, PrintWriter printWriter) {
		try {
			Event event = generalUnmarshalling(Event.class, message);
			log.info("MESSAGE: " + message);
			log.info("STATUS: " + event.getSubscription().get(0).getStatus());
			log.info("URN: " + event.getUrn());
			if (event.getSubscription() == null || event.getSubscription().get(0).getStatus() == null) {
				File xmlFile = new File(eventForCheckState);
				Event eventForCheckState = generalUnmarshalling(Event.class, xmlFile);
				eventForCheckState.setUrn(event.getUrn());
				StringWriter writer = new StringWriter();
				Marshaller marshaller = generalMarshalling(Event.class);
				marshaller.marshal(eventForCheckState, writer);
				log.info(String.valueOf(writer));
				printWriter.println(String.valueOf(writer));
			}

			if (event.getSubscription() != null && event.getSubscription().get(0).getStatus() != null) {
				log.info(event.getVariableName());
				Optional<SubscribeEvent> eventFromDB = eventRepository.findById(event.getSubscription().get(0).getVariableName());
				if (eventFromDB.isPresent()) {
					File xmlFile = new File(eventForSub);
					Event eventForSend = generalUnmarshalling(Event.class, xmlFile);
					SubscribeEvent subscribeEvent = eventRepository.findByKey(eventFromDB.get().getKey());

					//Заполняем поля значениями из сообщения
					VariableName variableName = new VariableName();
					eventForSend.setUrl(event.getUrl()); // положили url
					eventForSend.setKey(eventFromDB.get().getKey()); // положили key

					List<VariableName> variables = new ArrayList<VariableName>();
					variableName.setValue(subscribeEvent.getVariableName());
					variables.add(variableName);
					eventForSend.setPropertySet(variables);// положили value

					//маршаллинг
					StringWriter writer = new StringWriter();
					Marshaller marshaller = generalMarshalling(Event.class);
					marshaller.marshal(eventForSend, writer);

					subscribeEvent.setSubscribed(true);
					eventRepository.save(subscribeEvent);
					printWriter.println(String.valueOf(writer));
				}
				if(event.getPropertySet() == null){
					//Остановка потока с отправкой сообщений с событиями, если в сообщение пришло лишь одно поле
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Map<String, String> unmarshallGet(String message, PrintWriter writer) {
		try {
			Get get = generalUnmarshalling(Get.class, message);
			Map<String, String> map = new HashMap<>();
			String url = get.getUrl();
			String uuid = get.getUuid();
			if (uuid.split(":").length == 2) {
				UnmarshallServiceDevice(url, uuid, writer);
			} else
				UnmarshallDevice(url, uuid, writer);
			map.put("uuid", get.getUuid());
			return map;
		} catch (Exception e) {
			e.printStackTrace();
			return new HashMap<>();
		}
	}

	@PostConstruct
	public void runBroadcastClient() {

		Thread broadcastClient = new Thread(() -> {
			createConnection();
		});
		broadcastClient.start();

	}

	List<SubscribeEvent> events;
	@PostConstruct
	public void subscribeOnEvent(){
		Thread client = new Thread(() -> {
			events = eventRepository.findAll();
			createUDPConnection();
		});
		client.start();
	}

	@PostConstruct
	public void occurredError(){
		Thread client = new Thread(() -> fault("30000"));
		client.start();
	}

	public void fault(String code){
		try(DatagramSocket socket = new DatagramSocket(errorPort, InetAddress.getByName(broadcastIp))){
			Sender sender = new Sender(broadcastIp, errorPort);
			while(notSolved){
				DatagramPacket datagramPacket = new DatagramPacket(buffer, buffer.length);
				Error error = new Error();
				error.setUrn("KSK-MSUEV-04b-003d");
				error.setUrl(host + ":" + port);
				Describe describe = new Describe();
				describe.setCode(code);
				
				LocalDateTime now = LocalDateTime.now();
		        DateTimeFormatter formatterDate = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		        DateTimeFormatter formatterTime = DateTimeFormatter.ofPattern("HH:mm:ss");		       
				describe.setTime(now.format(formatterTime));
				describe.setDate(now.format(formatterDate));
				
				error.setDescribe(describe);
				StringWriter writer = new StringWriter();
				Marshaller marshaller = generalMarshalling(Error.class);
				marshaller.marshal(error, writer);

				sender.sendMessage(writer.toString());
				socket.receive(datagramPacket);
				String message = new String(datagramPacket.getData(), 0, datagramPacket.getLength());
				Thread.sleep(5000);
			}
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}

	long messageCount = 62280;
	public void createUDPConnection(){
		try (DatagramSocket socket = new DatagramSocket(port, InetAddress.getByName(broadcastIp));) {
			// Создаем сокет для прослушивания на указанном порту

			System.out.println("Listening for broadcast messages on port " + port + "...");
			Sender sender = new Sender(broadcastIp, port);

			while (true) {
				// Создаем пакет для приема данных

				for (SubscribeEvent event : events) {

					Event eventData = new Event();
					eventData.setUrn("KSK-MSUEV-04b-003d:2");
					eventData.setKey(++messageCount + "");

					DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
					DocumentBuilder db = dbf.newDocumentBuilder();
					Document document = db.newDocument();
					Marshaller marshaller =  generalMarshalling(Event.class);
					marshaller.marshal(eventData, document);

					try {
						Element newNode = document.createElement(event.getVariableName());
						Element valueNode = document.createElement("struct");
						Element structDataNode = document.createElement("DATA");
						Element structStatusNode = document.createElement("STATUS");
						Element dataValueNode = document.createElement("value");
						Element statusValueNode = document.createElement("value");
						
						dataValueNode.setTextContent("55");
						statusValueNode.setTextContent("NORMAL");
						newNode.appendChild(valueNode);

						valueNode.appendChild(structDataNode);
						valueNode.appendChild(structStatusNode);
						
						structDataNode.appendChild(dataValueNode);
						structStatusNode.appendChild(statusValueNode);

						document.getDocumentElement().getElementsByTagName("propertySet").item(0).appendChild(newNode);

						StringWriter writer = new StringWriter();
						TransformerFactory tf = TransformerFactory.newInstance();
						Transformer transformer = tf.newTransformer();
						transformer.transform(new DOMSource(document), new StreamResult(writer));

						sender.sendMessage(writer.toString());

					} catch(Exception e) {
						e.printStackTrace();
					}

				}
				Thread.sleep(5000);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void UnmarshallDevice(String url, String uuid, PrintWriter printWriter) {

		try {
			File xmlFile = new File(deviceFilePath);
			Root root = generalUnmarshalling(Root.class, xmlFile);
			Marshaller marshaller = generalMarshalling(Root.class);
			StringWriter writer = new StringWriter();
			marshaller.marshal(root, writer);
			printWriter.println(String.valueOf(writer));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public File getServiceType(String uuid){

		String serviceId = uuid.split(":")[1];
		File xmlFile = switch (serviceId) {
            case "1" -> new File(serviceMapFilePath);
            case "3" -> new File(serviceConfigFilePath);
            case "WEB" -> new File(serviceWebFilePath);
            default -> new File(serviceFilePath);
        };
		return xmlFile;
	}

	public void UnmarshallServiceDevice(String url, String uuid, PrintWriter printWriter) {
		try {
			File xmlFile = getServiceType(uuid);
			Scpd scpd = generalUnmarshalling(Scpd.class, xmlFile);
			StringWriter writer = new StringWriter();
			Marshaller marshaller = generalMarshalling(Scpd.class);
			scpd.setScpd_urn(uuid);
			marshaller.marshal(scpd, writer);
			printWriter.println(String.valueOf(writer));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void sendMessage(String url, StringWriter message) {
		String[] address = url.split(":");
		String ip = address[0];
		int port = Integer.valueOf(address[1]);
		Sender sender = new Sender(ip, port);
		sender.sendMessage(String.valueOf(message));
	}

}
