package org.skduemu.service;

import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.treewalk.TreeWalk;
import org.eclipse.jgit.treewalk.filter.TreeFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Service
public class FilesService {

    private final Logger log = LoggerFactory.getLogger(this.getClass().getName());

    @Autowired
    private GitService gitService;

    @Value("${repositoryLocalPath:/var/www}") // C:/git/skdu-scripts
    private String repositoryLocalPath;

    public ResponseEntity<?> getFilenames() {
        List<String> files = new ArrayList<>();
        if (gitService.gitPull().getStatusCode().value() == 200) {
            File file = gitService.getFileFromFilePath(repositoryLocalPath);
            try (Git git = Git.open(file)) {
                Repository repository = git.getRepository();
                TreeWalk treeWalk = new TreeWalk(repository);
                treeWalk.addTree(repository.resolve("HEAD^{tree}"));
                treeWalk.setRecursive(true);
                treeWalk.setFilter(TreeFilter.ALL);
                while (treeWalk.next()) {
                    files.add(treeWalk.getPathString());
                }
                treeWalk.close();
                log.info("treewalk closed");
            } catch (Exception e) {
                log.info(e.getMessage());
            }
            return new ResponseEntity<>(files, HttpStatus.OK);
        }
        return new ResponseEntity<>(files, HttpStatus.OK);
    }

    public ResponseEntity<Resource> getFile(String name){
        try{
            File file = gitService.getFileFromFilePath(repositoryLocalPath + "/" + name);
            Resource resource = new UrlResource(file.toPath().toUri());
            return new ResponseEntity<>(resource, HttpStatus.OK);
        }
        catch (Exception e){
            log.info(e.getMessage());
            return new ResponseEntity<>(null, HttpStatus.OK);
        }
    }
}
