package org.skduemu.service;

import org.eclipse.jgit.api.CloneCommand;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.PullResult;
import org.eclipse.jgit.merge.MergeStrategy;
import org.eclipse.jgit.transport.CredentialsProvider;
import org.eclipse.jgit.transport.UsernamePasswordCredentialsProvider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.io.File;

@Service
public class GitService {

    private final Logger log = LoggerFactory.getLogger(this.getClass().getName());

    @Value("${gitLab.branch:main}")
    private String branch;

    @Value("${gitLab.url:https://gitlab.mifors.com/ksk/skdu-scripts}")
    private String repUrl;

    @Value("${gitLab.token:zEJmFifi_3c5vUTWZfXc}")
    private String token;

//    @Value("${repositoryLocalPath:/var/www/skdu-scripts}") C:\git\skdu-scripts
//    private String repositoryLocalPath;

    @Value("${repositoryLocalPath:C:/git/skdu-scripts}")
    private String repositoryLocalPath;

    public File getFileFromFilePath(String path){
        return new File(path);
    }

    private void authorization(){
        CredentialsProvider credentialsProvider = new UsernamePasswordCredentialsProvider("PRIVATE_TOKEN", token);
        File file = getFileFromFilePath(repositoryLocalPath);
        try(Git git = Git.open(file)){
            PullResult pullResult = git.pull().setCredentialsProvider(credentialsProvider).call();
        }
        catch (Exception e){
            log.info(e.getMessage());
        }
    }

    public ResponseEntity<?> checkoutToBranch(String branchName, boolean createNewBranch, boolean forceRefUpdate){
        authorization();
        File file = getFileFromFilePath(repositoryLocalPath);
        try(Git git = Git.open(file)){
            git.checkout()
                    .setCreateBranch(createNewBranch)
                    .setName(branchName)
                    .setForceRefUpdate(forceRefUpdate)
                    .call();
            branch = branchName;
        }
        catch (Exception e){
            log.info(e.getMessage());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }

    public ResponseEntity<?> gitPull() {
        File file = getFileFromFilePath(repositoryLocalPath);
        CredentialsProvider credentialsProvider = new UsernamePasswordCredentialsProvider("PRIVATE_TOKEN", token);
        try (Git git = Git.open(file)){
            PullResult pull = git.pull()
                    .setCredentialsProvider(credentialsProvider)
                    .setRemoteBranchName(branch)
                    .setStrategy(MergeStrategy.THEIRS)
                    .call();
        }
        catch (Exception e){
            log.info(e.getMessage());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        log.info("pulled");
        return new ResponseEntity<>(HttpStatus.OK);
    }

    public ResponseEntity<?> cloneRepo(){
        CredentialsProvider credentialsProvider = new UsernamePasswordCredentialsProvider("PRIVATE_TOKEN", token);
        try{
            CloneCommand cloneCommand = new CloneCommand();
            cloneCommand.setCredentialsProvider(credentialsProvider);
            cloneCommand.setURI(repUrl);
            cloneCommand.setDirectory(getFileFromFilePath(repositoryLocalPath));
            cloneCommand.setBranch(branch);
            cloneCommand.call();
        }
        catch (Exception e){
            log.info(e.getMessage());
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(HttpStatus.OK);
    }
}
