package org.skduemu.service;

import org.skduemu.utils.ModbusUtils;
import org.skduemu.utils.ModbusWriteUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

@Service
public class ModbusService {

    @Autowired
    private ModbusWriteUtils modbusWriteUtils;

    @Autowired
    private ModbusUtils modbusUtils;

    @Value("${coils:3}")
    private int countOfCoils;

    @Value("${inputRegisters:3}")
    private int countOfInputRegisters;

    @Value("${holdingRegisters:4}")
    private int countOfHoldingRegisters;

    private final Logger log = LoggerFactory.getLogger(this.getClass().getName());

    public String readCoilStatus(int startOffset, int endOffset){
        int length = endOffset - startOffset;
        short[] coils = new short[length];
        try{
            for(int i = 0; i < length; i++){
                coils[i] = (short) (modbusUtils.readCoilStatus(1, i) ? 1 : 0);
            }
        }
        catch (Exception e){
            e.printStackTrace();
        }
        return Arrays.toString(coils);
    }

    public void mqttToModbus(List<Short> sdata){
        try {
            short[] data = new short[sdata.size()];
            for(int i = 0; i < data.length; i++){
                data[i] = sdata.get(i);
            }
            boolean result = modbusWriteUtils.writeRegisters(1, 1, data);
        }
        catch(Exception e){
            log.info(e.getLocalizedMessage());
        }
    }

}
