package org.skduemu.utils;

import com.serotonin.modbus4j.ModbusFactory;
import com.serotonin.modbus4j.ModbusMaster;
import com.serotonin.modbus4j.exception.ErrorResponseException;
import com.serotonin.modbus4j.exception.ModbusInitException;
import com.serotonin.modbus4j.exception.ModbusTransportException;
import com.serotonin.modbus4j.ip.IpParameters;
import com.serotonin.modbus4j.locator.BaseLocator;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
public class ModbusUtils {

    @Value("${modbusIp:192.168.1.28}")
   // @Value("${modbusIp:217.14.197.50}")
    private String host;

    @Value("${modbusPort:502}")
    private int modbusPort;

    static ModbusFactory modbusFactory;
    static {
        if (modbusFactory == null) {
            modbusFactory = new ModbusFactory();
        }
    }

    /**
     * Get Master
     *
     * @return
     * @throws ModbusInitException
     */
    public ModbusMaster getMaster() throws ModbusInitException {
        IpParameters params = new IpParameters();
        params.setHost(host);
        params.setPort(modbusPort);
        //
        // ModbusFactory.creatertumaster (WAPPER); // RTU protocol
        // ModbusFactory.createudpmaster (Params); // UDP protocol
        // ModbusFactory.createasciimaster (wrapper); // ASCII protocol
        ModbusMaster master = modbusFactory.createTcpMaster(params, false);// TCP protocol
        master.init();

        return master;
    }

    /**
     * Read [01 Coil Status 0x] type switch data
     *
     * @param slaveId
     *            slaveId
     * @param offset
     *            Location
     * @Return Read the value
     * @throws ModbusTransportException
     * Abnormal
     * @throws ErrorResponseException
     * Abnormal
     * @throws ModbusInitException
     * Abnormal
     */
    public Boolean readCoilStatus(int slaveId, int offset)
            throws ModbusTransportException, ErrorResponseException, ModbusInitException {
        // 01 Coil Status
        BaseLocator<Boolean> loc = BaseLocator.coilStatus(slaveId, offset);
        Boolean value = getMaster().getValue(loc);
        return value;
    }
}
