package org.skduemu.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.serotonin.modbus4j.ModbusFactory;
import com.serotonin.modbus4j.ModbusMaster;
import com.serotonin.modbus4j.exception.ModbusInitException;
import com.serotonin.modbus4j.exception.ModbusTransportException;
import com.serotonin.modbus4j.ip.IpParameters;
import com.serotonin.modbus4j.msg.ModbusResponse;
import com.serotonin.modbus4j.msg.WriteRegistersRequest;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
public class ModbusWriteUtils {

    //@Value("${modbusIp:192.168.1.87}")
    @Value("${modbusIp:192.168.1.28}")
    private String host;

    @Value("${modbusPort:502}")
    private int modbusPort;


    static Log log = LogFactory.getLog(ModbusWriteUtils.class);
    /**
     * factory.
     */
    static ModbusFactory modbusFactory;
    static {
        if (modbusFactory == null) {
            modbusFactory = new ModbusFactory();
        }
    }

    /**
     * Get TCPMaster
     *
     * @return
     * @throws ModbusInitException
     */
    public ModbusMaster getMaster() throws ModbusInitException {
        IpParameters params = new IpParameters();
        params.setHost(host);
        params.setPort(modbusPort);

        ModbusMaster tcpMaster = modbusFactory.createTcpMaster(params, false);
        tcpMaster.init();

        return tcpMaster;
    }

    public boolean writeRegisters(int slaveId, int startOffset, short[] sdata)
            throws ModbusTransportException, ModbusInitException {
        // Get Master
        ModbusMaster tcpMaster = getMaster();
        // Create a request object
        WriteRegistersRequest request = new WriteRegistersRequest(slaveId, startOffset, sdata);
        // Send a request and get a response object
        ModbusResponse response = tcpMaster.send(request);
        if (response.isException()) {
            log.error(response.getExceptionMessage());
            return false;
        } else {
            return true;
        }
    }
}
